package oracle.iam.sample.notification;

import static oracle.iam.identity.utils.Constants.DISPLAYNAME;
import static oracle.iam.identity.utils.Constants.MLS_BASE_VALUE;
import static oracle.iam.identity.utils.Constants.USERKEY;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import oracle.iam.identity.rolemgmt.api.RoleManager;
import oracle.iam.identity.rolemgmt.vo.Role;
import oracle.iam.identity.usermgmt.api.UserManager;
import oracle.iam.identity.vo.Identity;
import oracle.iam.notification.api.NotificationService;
import oracle.iam.notification.impl.NotificationEventResolver;
import oracle.iam.notification.vo.NotificationAttribute;
import oracle.iam.platform.Platform;

public class RoleGrantNotificationEventResolver implements NotificationEventResolver{

	private static String ROLEKEY = "role_key";

	/* (non-Javadoc)
	 * @see oracle.iam.notification.impl.NotificationEventResolver#getAvailableData(
	 * 	java.lang.String, java.util.Map)
	 */
	@Override
	public List<NotificationAttribute> getAvailableData(String eventType,
		Map<String, Object> params) throws Exception {
		List<NotificationAttribute> list = new ArrayList<NotificationAttribute>();
		return list;
	}

	/* (non-Javadoc)
	 * @see oracle.iam.notification.impl.NotificationEventResolver#getReplacedData(
	 * 	java.lang.String, java.util.Map)
	 */
	@Override
	public HashMap<String, Object> getReplacedData(String eventType,
		Map<String, Object> params) throws Exception {

		HashMap<String, Object> resolvedData = new HashMap<String, Object>();
		UserManager usrMgr = Platform.getService(UserManager.class);
		String userKey = (String) params.get(USERKEY);
		String roleKey = (String) params.get(ROLEKEY);

		// Mapping token with their actual value for user attributes.
		if (userKey != null) {
	      
			NotificationService notificationService = 
				Platform.getService(NotificationService.class);
			List<NotificationAttribute> notificationAttributes = 
				notificationService.getStaticData(eventType);
	      
			/* 
			 * Configuring user attributes required to be returned in the search 
			 * aligning with attributes being showed as 'Available Data' in the 
			 * notification template for this event.
	      		 */
			Set<String> userRetAttrs = new HashSet<String>();
			for (NotificationAttribute notificationAttribute : 
				notificationAttributes.get(0).getSubtree()) {
				userRetAttrs.add(notificationAttribute.getName());
			}
	      
			//Getting values for the attributes using userKey
			Identity user = usrMgr.getDetails(userKey, userRetAttrs ,false);
			HashMap<String, Object> userAttributes = user.getAttributes();
	      
			/*
			 * Creating map containing mapping between tokens available for template 
			 * to their actual values 
			 */
			String key = null;
			for (Map.Entry<String, Object>  entry : userAttributes.entrySet()) {
				key = entry.getKey();
				if (key != null) {
					if ((entry.getValue() instanceof java.util.Map) && 
						(key.equalsIgnoreCase(DISPLAYNAME))) {
						key = key.replace(' ', '_');
						resolvedData.put(key, 
							((HashMap)entry.getValue()).get(MLS_BASE_VALUE));
					} else {
						key = key.replace(' ', '_');
						resolvedData.put(key, entry.getValue());
					}
				}
			}
		}
	    
		// Mapping token associated with Role entity in the template with its actual value 
		if (roleKey != null) {
			RoleManager roleMgr = Platform.getService(RoleManager.class);
			Set<String> roleRetAttrs = new HashSet<String>();
	    	
			// Setting return attributes to return only Role Name
			roleRetAttrs.add("Role Name");
	    	
			Role role = roleMgr.getDetails(roleKey, roleRetAttrs);
			resolvedData.put("UGP_ROLENAME", role.getName());
		}
		return resolvedData;
	}
}
